@@ -54,6 +54,7 @@
 
 -define(MAX_BUF_SIZE, 64*1024). %% 64kb
 -define(TIMEOUT, 60000). %% 1 minute
+-define(TCP_ACTIVE, 500).
 -define(NONCE_LIFETIME, 60*1000*1000). %% 1 minute (in usec)
 -define(SERVER_NAME, <<"P1 STUN library">>).
 
@@ -127,10 +128,10 @@ init([Sock, Opts]) ->
 		    State = prepare_state(Opts, Sock, Addr, SockMod),
 		    case maybe_starttls(Sock, SockMod, Opts) of
 			{ok, NewSock} ->
-			    inet:setopts(Sock, [{active, once}]),
 			    TRef = erlang:start_timer(?TIMEOUT, self(), stop),
-			    {ok, session_established,
-			     State#state{tref = TRef, sock = NewSock}};
+			    NewState = State#state{sock = NewSock, tref = TRef},
+			    activate_socket(NewState),
+			    {ok, session_established, NewState};
 			{error, Reason} ->
 			    {stop, Reason}
 		    end;
@@ -166,6 +167,9 @@ handle_info({tcp, _Sock, TLSData}, StateName,
 handle_info({tcp, _Sock, Data}, StateName, State) ->
     NewState = update_shaper(State, Data),
     process_data(StateName, NewState, Data);
+handle_info({tcp_passive, _Sock}, StateName, State) ->
+    activate_socket(State),
+    {next_state, StateName, State};
 handle_info({tcp_closed, _Sock}, _StateName, State) ->
     ?LOG_INFO("Connection reset by peer"),
     {stop, normal, State};
@@ -221,6 +225,7 @@ process(#state{auth = anonymous} = State,
     Resp = prepare_response(State, Msg),
     R = Resp#stun{class = error,
 		  'ERROR-CODE' = stun_codec:error(401)},
+    run_hook(protocol_error, State, R),
     send(State, R);
 process(#state{auth = user} = State,
 	#stun{class = request,
@@ -241,6 +246,7 @@ process(#state{auth = user} = State,
 	    case (State#state.auth_fun)(User, Realm) of
 		<<"">> ->
 		    ?LOG_NOTICE("Failed long-term STUN/TURN authentication"),
+		    run_hook(protocol_error, State, R),
 		    send(NewState, R);
 		Pass0 ->
 		    {Pass, IsExpired} = check_expired_tag(Pass0),
@@ -252,6 +258,7 @@ process(#state{auth = user} = State,
 			false ->
 			    ?LOG_NOTICE("Failed long-term STUN/TURN "
 					"authentication"),
+			    run_hook(protocol_error, State, R),
 			    send(NewState, R)
 		    end
 	    end;
@@ -262,6 +269,7 @@ process(#state{auth = user} = State,
 			  'ERROR-CODE' = stun_codec:error(438),
 			  'REALM' = State#state.realm,
 			  'NONCE' = NewNonce},
+	    run_hook(protocol_error, State, R),
 	    send(State#state{nonces = NewNonces}, R)
     end;
 process(State, #stun{class = request,
@@ -272,12 +280,14 @@ process(State, #stun{class = request,
     Resp = prepare_response(State, Msg),
     R = Resp#stun{class = error,
 		  'ERROR-CODE' = stun_codec:error(401)},
+    run_hook(protocol_error, State, R),
     send(State, R);
 process(State, #stun{class = request} = Msg) ->
     ?LOG_NOTICE("Rejecting malformed request"),
     Resp = prepare_response(State, Msg),
     R = Resp#stun{class = error,
 		  'ERROR-CODE' = stun_codec:error(400)},
+    run_hook(protocol_error, State, R),
     send(State, R);
 process(State, #stun{class = indication,
 		     method = ?STUN_METHOD_SEND} = Msg) ->
@@ -298,6 +308,7 @@ process(State, #stun{class = request, unsupported = [_|_] = Unsupported} = Msg,
     R = Resp#stun{class = error,
 		  'UNKNOWN-ATTRIBUTES' = Unsupported,
 		  'ERROR-CODE' = stun_codec:error(420)},
+    run_hook(protocol_error, State, R),
     send(State, R, Secret);
 process(State, #stun{class = request,
 		     method = ?STUN_METHOD_BINDING} = Msg, Secret,
@@ -319,6 +330,7 @@ process(#state{use_turn = false} = State,
     ?LOG_NOTICE("Rejecting TURN request: TURN is disabled"),
     Resp = prepare_response(State, Msg),
     R = Resp#stun{class = error, 'ERROR-CODE' = stun_codec:error(405)},
+    run_hook(protocol_error, State, R),
     send(State, R, Secret);
 process(State, #stun{class = request,
 		     method = ?STUN_METHOD_ALLOCATE} = Msg,
@@ -333,6 +345,7 @@ process(State, #stun{class = request,
 	_ when IsExpired ->
 	    ?LOG_NOTICE("Rejecting request: credentials expired"),
 	    R = Resp#stun{class = error, 'ERROR-CODE' = stun_codec:error(401)},
+	    run_hook(protocol_error, State, R),
 	    send(State, R);
 	_ ->
 	    Opts = [{sock, State#state.sock},
@@ -366,16 +379,19 @@ process(State, #stun{class = request,
 		    ?LOG_NOTICE("Rejecting request: Allocation quota reached"),
 		    R = Resp#stun{class = error,
 				  'ERROR-CODE' = stun_codec:error(486)},
+		    run_hook(protocol_error, State, R),
 		    send(State, R, Secret);
 		{error, stale} ->
 		    ?LOG_NOTICE("Rejecting request: Stale nonce"),
 		    R = Resp#stun{class = error,
 				  'ERROR-CODE' = stun_codec:error(438)},
+		    run_hook(protocol_error, State, R),
 		    send(State, R);
 		{error, Reason} ->
 		    ?LOG_ERROR("Cannot start TURN session: ~s", [Reason]),
 		    R = Resp#stun{class = error,
 				  'ERROR-CODE' = stun_codec:error(500)},
+		    run_hook(protocol_error, State, R),
 		    send(State, R, Secret)
 	    end
     end;
@@ -395,6 +411,7 @@ process(State, #stun{class = request} = Msg, Secret, _IsExpired) ->
     ?LOG_NOTICE("Rejecting request: Method not allowed"),
     Resp = prepare_response(State, Msg),
     R = Resp#stun{class = error, 'ERROR-CODE' = stun_codec:error(405)},
+    run_hook(protocol_error, State, R),
     send(State, R, Secret).
 
 process_data(NextStateName, #state{buf = Buf} = State, Data) ->
@@ -415,6 +432,8 @@ process_data(NextStateName, #state{buf = Buf} = State, Data) ->
 	    {stop, normal, State}
     end.
 
+update_shaper(#state{shaper = none} = State, _Data) ->
+    State;
 update_shaper(#state{shaper = Shaper} = State, Data) ->
     {NewShaper, Pause} = stun_shaper:update(Shaper, size(Data)),
     if Pause > 0 ->
@@ -460,6 +479,7 @@ route_on_turn(State, Msg, Pass) ->
 		    Resp = prepare_response(State, Msg),
 		    R = Resp#stun{class = error,
 				  'ERROR-CODE' = stun_codec:error(437)},
+		    run_hook(protocol_error, State, R),
 		    send(State, R, Pass);
 		_ ->
 		    State
@@ -598,6 +618,8 @@ prepare_state(Opts, Sock, Peer, SockMod) when is_list(Opts) ->
 		 ({protocol_options, _}, State) -> State;
 		 ({tls, _}, State) -> State;
 		 (tls, State) -> State;
+		 ({proxy_protocol, _}, State) -> State;
+		 (proxy_protocol, State) -> State;
 		 ({sock_peer_name, _}, State) -> State;
 		 ({session_id, _}, State) -> State;
 		 (Opt, State) ->
@@ -628,13 +650,14 @@ prepare_addr(T) when is_tuple(T) ->
 	    {error, einval}
     end.
 
+activate_socket(#state{sock = Sock, sock_mod = gen_tcp, shaper = none}) ->
+    inet:setopts(Sock, [{active, ?TCP_ACTIVE}]);
+activate_socket(#state{sock = Sock, sock_mod = SockMod, shaper = none}) ->
+    SockMod:setopts(Sock, [{active, ?TCP_ACTIVE}]);
+activate_socket(#state{sock = Sock, sock_mod = gen_tcp}) ->
+    inet:setopts(Sock, [{active, once}]);
 activate_socket(#state{sock = Sock, sock_mod = SockMod}) ->
-    case SockMod of
-	gen_tcp ->
-	    inet:setopts(Sock, [{active, once}]);
-	_ ->
-	    SockMod:setopts(Sock, [{active, once}])
-    end.
+    SockMod:setopts(Sock, [{active, once}]).
 
 cancel_timer(undefined) ->
     ok;
@@ -807,14 +830,16 @@ run_hook(HookName,
 		sock_mod = SockMod,
 		hook_fun = HookFun},
 	 #stun{'USERNAME' = User,
-	       'REALM' = Realm} = Msg)
+	       'REALM' = Realm,
+	       'ERROR-CODE' = Reason} = Msg)
   when is_function(HookFun) ->
     Info = #{id => ID,
 	     user => User,
 	     realm => Realm,
 	     client => Client,
 	     transport => stun_logger:encode_transport(SockMod),
-	     version => stun_codec:version(Msg)},
+	     version => stun_codec:version(Msg),
+	     reason => Reason},
     ?LOG_DEBUG("Running '~s' hook", [HookName]),
     try HookFun(HookName, Info)
     catch _:Err -> ?LOG_ERROR("Hook '~s' failed: ~p", [HookName, Err])
