@@ -5,7 +5,7 @@
 %%% Created :  7 Aug 2009 by Evgeniy Khramtsov <ekhramtsov@process-one.net>
 %%%
 %%%
-%%% Copyright (C) 2002-2021 ProcessOne, SARL. All Rights Reserved.
+%%% Copyright (C) 2002-2022 ProcessOne, SARL. All Rights Reserved.
 %%%
 %%% Licensed under the Apache License, Version 2.0 (the "License");
 %%% you may not use this file except in compliance with the License.
@@ -252,9 +252,11 @@ dec_attr(?STUN_ATTR_XOR_PEER_ADDRESS, Val, Msg) ->
     Tail = Msg#stun.'XOR-PEER-ADDRESS',
     Msg#stun{'XOR-PEER-ADDRESS' = [AddrPort|Tail]};
 dec_attr(?STUN_ATTR_REQUESTED_ADDRESS_FAMILY, Val, Msg) ->
-    Family = case Val of
-		 <<1:8, _:3/binary>> -> ipv4;
-		 <<2:8, _:3/binary>> -> ipv6
+    <<FamilyInt, _:3/binary>> = Val,
+    Family = case FamilyInt of
+		 1 -> ipv4;
+		 2 -> ipv6;
+		 _ -> unknown
 	     end,
     Msg#stun{'REQUESTED-ADDRESS-FAMILY' = Family};
 dec_attr(?STUN_ATTR_REQUESTED_TRANSPORT, Val, Msg) ->
