@@ -5,7 +5,7 @@
 %%% Created : 19 Jul 2020 by Holger Weiss <holger@zedat.fu-berlin.de>
 %%%
 %%%
-%%% Copyright (C) 2020-2021 ProcessOne, SARL. All Rights Reserved.
+%%% Copyright (C) 2020-2022 ProcessOne, SARL. All Rights Reserved.
 %%%
 %%% Licensed under the Apache License, Version 2.0 (the "License");
 %%% you may not use this file except in compliance with the License.
@@ -144,7 +144,7 @@ add_metadata(Meta) ->
 filter(#{meta := #{domain := [stun | _]},
 	 msg := {report, #{verbatim := Msg}}} = Event,
        _Extra) ->
-    Event#{msg => Msg};
+    Event#{msg := Msg};
 filter(#{meta := #{domain := [stun | _],
 		   stun_transport := _Transport} = Meta,
 	 msg := {Format, Args}} = Event,
@@ -155,7 +155,7 @@ filter(#{meta := #{domain := [stun | _],
 	       _ ->
 		   io_lib:format(Format, Args)
 	   end,
-    Event#{msg => format_msg(Text, Meta)};
+    Event#{msg := format_msg(Text, Meta)};
 filter(_Event, _Extra) ->
     ignore.
 -endif.
@@ -168,7 +168,7 @@ set_metadata(SubDomain, SockMod, ID, Addr) ->
 -spec make_id() -> binary().
 make_id() ->
     iolist_to_binary(
-      io_lib:format("~.36b", [erlang:unique_integer([positive])])).
+      io_lib:format("~12.36.0b", [p1_rand:uniform(16#4000000000000000)])).
 
 -spec encode_addr({inet:ip_address(), inet:port_number()} | inet:ip_address())
       -> iolist().
