@@ -43,15 +43,21 @@ start_link() ->
 %%% Supervisor callbacks
 %%%===================================================================
 init([]) ->
-    StunTmpSup = {stun_tmp_sup, {stun_tmp_sup, start_link, []},
-		  permanent, infinity, supervisor, [stun_tmp_sup]},
-    TurnTmpSup = {turn_tmp_sup, {turn_tmp_sup, start_link, []},
-		  permanent, infinity, supervisor, [turn_tmp_sup]},
-    TurnSM = {turn_sm, {turn_sm, start_link, []},
-	      permanent, 2000, worker, [turn_sm]},
-    StunListen = {stun_listener, {stun_listener, start_link, []},
-		  permanent, 2000, worker, [stun_listener]},
-    {ok, {{one_for_one, 10, 1}, [TurnSM, StunTmpSup, TurnTmpSup, StunListen]}}.
+    SupFlags = #{intensity => 10,
+		 period => 1},
+    ChildSpecs = [#{id => stun_listener_sup,
+		    type => supervisor,
+		    start => {stun_listener_sup, start_link, []}},
+		  #{id => stun_tmp_sup,
+		    type => supervisor,
+		    start => {stun_tmp_sup, start_link, []}},
+		  #{id => turn_tmp_sup,
+		    type => supervisor,
+		    start => {turn_tmp_sup, start_link, []}},
+		  #{id => turn_sm,
+		    shutdown => 2000,
+		    start => {turn_sm, start_link, []}}],
+    {ok, {SupFlags, ChildSpecs}}.
 
 %%%===================================================================
 %%% Internal functions
