@@ -5,7 +5,7 @@
 %%% Created : 23 Aug 2009 by Evgeniy Khramtsov <ekhramtsov@process-one.net>
 %%%
 %%%
-%%% Copyright (C) 2002-2021 ProcessOne, SARL. All Rights Reserved.
+%%% Copyright (C) 2002-2022 ProcessOne, SARL. All Rights Reserved.
 %%%
 %%% Licensed under the Apache License, Version 2.0 (the "License");
 %%% you may not use this file except in compliance with the License.
@@ -56,9 +56,9 @@
 
 -type addr() :: {inet:ip_address(), inet:port_number()}.
 -type subnet() :: {inet:ip4_address(), 0..32} | {inet:ip6_address(), 0..128}.
--type blacklist() :: [subnet()].
+-type accesslist() :: [subnet()].
 
--export_type([blacklist/0]).
+-export_type([accesslist/0]).
 
 -record(state,
 	{sock_mod = gen_udp                :: gen_udp | gen_tcp | fast_tls,
@@ -83,7 +83,8 @@
 	 last_pkt = <<>>                   :: binary(),
 	 seq = 1                           :: non_neg_integer(),
 	 life_timer                        :: reference() | undefined,
-	 blacklist = []                    :: blacklist(),
+	 blacklist = []                    :: accesslist(),
+	 whitelist = []                    :: accesslist(),
 	 hook_fun                          :: function() | undefined,
 	 session_id                        :: binary(),
 	 rcvd_bytes = 0                    :: non_neg_integer(),
@@ -120,6 +121,7 @@ init([Opts]) ->
     SockMod = proplists:get_value(sock_mod, Opts),
     HookFun = proplists:get_value(hook_fun, Opts),
     Blacklist = proplists:get_value(blacklist, Opts) ++ ?INITIAL_BLACKLIST,
+    Whitelist = proplists:get_value(whitelist, Opts),
     State = #state{sock_mod = SockMod,
 		   sock = proplists:get_value(sock, Opts),
 		   key = proplists:get_value(key, Opts),
@@ -131,7 +133,7 @@ init([Opts]) ->
 		   server_name = proplists:get_value(server_name, Opts),
 		   username = Username, realm = Realm, addr = AddrPort,
 		   session_id = ID, owner = Owner, hook_fun = HookFun,
-		   blacklist = Blacklist},
+		   blacklist = Blacklist, whitelist = Whitelist},
     stun_logger:set_metadata(turn, SockMod, ID, AddrPort, Username),
     MaxAllocs = proplists:get_value(max_allocs, Opts),
     if is_pid(Owner) ->
@@ -155,9 +157,10 @@ wait_for_allocate(#stun{class = request,
 			method = ?STUN_METHOD_ALLOCATE} = Msg,
 		  State) ->
     Family = case Msg#stun.'REQUESTED-ADDRESS-FAMILY' of
-		 undefined -> inet;
 		 ipv4 -> inet;
-		 ipv6 -> inet6
+		 ipv6 -> inet6;
+		 undefined -> inet;
+		 unknown -> unknown
 	     end,
     IsBlacklisted = blacklisted(State),
     Resp = prepare_response(State, Msg),
@@ -165,11 +168,13 @@ wait_for_allocate(#stun{class = request,
 	    ?LOG_NOTICE("Rejecting allocation request: no transport requested"),
 	    R = Resp#stun{class = error,
 			  'ERROR-CODE' = stun_codec:error(400)},
+	    run_hook(protocol_error, State, R),
 	    {stop, normal, send(State, R)};
        Msg#stun.'REQUESTED-TRANSPORT' == unknown ->
 	    ?LOG_NOTICE("Rejecting allocation request: unsupported transport"),
 	    R = Resp#stun{class = error,
 			  'ERROR-CODE' = stun_codec:error(442)},
+	    run_hook(protocol_error, State, R),
 	    {stop, normal, send(State, R)};
        Msg#stun.'DONT-FRAGMENT' == true ->
 	    ?LOG_NOTICE("Rejecting allocation request: dont-fragment not "
@@ -177,17 +182,26 @@ wait_for_allocate(#stun{class = request,
 	    R = Resp#stun{class = error,
 			  'UNKNOWN-ATTRIBUTES' = [?STUN_ATTR_DONT_FRAGMENT],
 			  'ERROR-CODE' = stun_codec:error(420)},
+	    run_hook(protocol_error, State, R),
+	    {stop, normal, send(State, R)};
+       Family == unknown ->
+	    ?LOG_NOTICE("Rejecting allocation request: unknown address family"),
+	    R = Resp#stun{class = error,
+			  'ERROR-CODE' = stun_codec:error(440)},
+	    run_hook(protocol_error, State, R),
 	    {stop, normal, send(State, R)};
        Family == inet6, State#state.relay_ipv6_ip == undefined ->
 	    ?LOG_NOTICE("Rejecting allocation request: IPv6 not supported"),
 	    R = Resp#stun{class = error,
 			  'ERROR-CODE' = stun_codec:error(440)},
+	    run_hook(protocol_error, State, R),
 	    {stop, normal, send(State, R)};
        IsBlacklisted ->
 	    ?LOG_NOTICE("Rejecting allocation request: Client address is "
 			"blacklisted"),
 	    R = Resp#stun{class = error,
 			  'ERROR-CODE' = stun_codec:error(403)},
+	    run_hook(protocol_error, State, R),
 	    {stop, normal, send(State, R)};
        true ->
 	    RelayIP = case Family of
@@ -217,6 +231,7 @@ wait_for_allocate(#stun{class = request,
 			       [format_error(Err)]),
 		    R = Resp#stun{class = error,
 				  'ERROR-CODE' = stun_codec:error(508)},
+		    run_hook(protocol_error, State, R),
 		    {stop, normal, send(State, R)}
 	    end
     end;
@@ -233,6 +248,7 @@ active(#stun{class = request,
     Resp = prepare_response(State, Msg),
     R = Resp#stun{class = error,
 		  'ERROR-CODE' = stun_codec:error(437)},
+    run_hook(protocol_error, State, R),
     {next_state, active, send(State, R)};
 active(#stun{class = request,
 	     'REQUESTED-ADDRESS-FAMILY' = ipv4,
@@ -242,6 +258,7 @@ active(#stun{class = request,
     Resp = prepare_response(State, Msg),
     R = Resp#stun{class = error,
 		  'ERROR-CODE' = stun_codec:error(443)},
+    run_hook(protocol_error, State, R),
     {next_state, active, send(State, R)};
 active(#stun{class = request,
 	     'REQUESTED-ADDRESS-FAMILY' = ipv6,
@@ -251,6 +268,7 @@ active(#stun{class = request,
     Resp = prepare_response(State, Msg),
     R = Resp#stun{class = error,
 		  'ERROR-CODE' = stun_codec:error(443)},
+    run_hook(protocol_error, State, R),
     {next_state, active, send(State, R)};
 active(#stun{class = request,
 	     method = ?STUN_METHOD_REFRESH} = Msg, State) ->
@@ -288,6 +306,7 @@ active(#stun{class = request,
 	    ?LOG_NOTICE("Rejecting permission creation request: ~s", [Txt]),
 	    R = Resp#stun{class = error,
 			  'ERROR-CODE' = Err},
+	    run_hook(protocol_error, State, R),
 	    {next_state, active, send(State, R)}
     end;
 active(#stun{class = indication,
@@ -315,6 +334,7 @@ active(#stun{class = request,
 			"to a different channel (~.16B)", [OldChannel]),
 	    R = Resp#stun{class = error,
 			  'ERROR-CODE' = stun_codec:error(400)},
+	    run_hook(protocol_error, State, R),
 	    {next_state, active, send(State, R)};
 	{{ok, {OldPeer, _}}, _} when Peer /= OldPeer ->
 	    ?LOG_NOTICE("Rejecting channel binding request: Channel already "
@@ -322,6 +342,7 @@ active(#stun{class = request,
 			[stun_logger:encode_addr(OldPeer)]),
 	    R = Resp#stun{class = error,
 			  'ERROR-CODE' = stun_codec:error(400)},
+	    run_hook(protocol_error, State, R),
 	    {next_state, active, send(State, R)};
 	{FindResult, _} ->
 	    case update_permissions(State, [Addr]) of
@@ -348,6 +369,7 @@ active(#stun{class = request,
 		    ?LOG_NOTICE("Rejecting channel binding request: ~s", [Txt]),
 		    R = Resp#stun{class = error,
 				  'ERROR-CODE' = Err},
+		    run_hook(protocol_error, State, R),
 		    {next_state, active, send(State, R)}
 	    end
     end;
@@ -358,6 +380,7 @@ active(#stun{class = request,
     Resp = prepare_response(State, Msg),
     R = Resp#stun{class = error,
 		  'ERROR-CODE' = stun_codec:error(400)},
+    run_hook(protocol_error, State, R),
     {next_state, active, send(State, R)};
 active(#turn{channel = Channel, data = Data}, State) ->
     case maps:find(Channel, State#state.channels) of
@@ -583,14 +606,22 @@ family_matches(_Addr1, _Addr2) ->
 blacklisted(#state{addr = {IP, _Port}} = State) ->
     blacklisted(State, [IP]).
 
-blacklisted(#state{blacklist = Blacklist}, IPs) ->
+blacklisted(#state{blacklist = Blacklist, whitelist = Whitelist}, IPs) ->
     lists:any(
       fun(IP) ->
 	      lists:any(
 		fun({Net, Mask}) ->
 			match_subnet(IP, Net, Mask)
 		end, Blacklist)
-      end, IPs).
+      end, IPs)
+          andalso not
+	      lists:any(
+		fun(IP) ->
+			lists:any(
+			  fun({Net, Mask}) ->
+				  match_subnet(IP, Net, Mask)
+			  end, Whitelist)
+		end, IPs).
 
 match_subnet({_, _, _, _} = IP,
 	     {_, _, _, _} = Net, Mask) ->
@@ -673,36 +704,44 @@ count_rcvd(#state{rcvd_bytes = RcvdSize,
     State#state{rcvd_bytes = RcvdSize + byte_size(Data),
 		rcvd_pkts = RcvdPkts + 1}.
 
-run_hook(HookName, #state{session_id = ID,
-			  username = User,
-			  realm = Realm,
-			  addr = Client,
-			  sock_mod = SockMod,
-			  hook_fun = HookFun} = State)
+run_hook(HookName, State) ->
+    run_hook(HookName, State, #stun{}).
+
+run_hook(HookName,
+	 #state{session_id = ID,
+		username = User,
+		realm = Realm,
+		addr = Client,
+		sock_mod = SockMod,
+		hook_fun = HookFun} = State,
+	 #stun{'ERROR-CODE' = Reason})
   when is_function(HookFun) ->
-    Info0 = #{id => ID,
+    Info1 = #{id => ID,
 	      user => User,
 	      realm => Realm,
 	      client => Client,
-	      transport => stun_logger:encode_transport(SockMod)},
-    Info = case {HookName, State} of
-	       {turn_session_start, _State} ->
-		   Info0;
-	       {turn_session_stop, #state{sent_bytes = SentBytes,
-					  sent_pkts = SentPkts,
-					  rcvd_bytes = RcvdBytes,
-					  rcvd_pkts = RcvdPkts}} ->
-		   Info0#{sent_bytes => SentBytes,
-			  sent_pkts => SentPkts,
-			  rcvd_bytes => RcvdBytes,
-			  rcvd_pkts => RcvdPkts,
-			  duration => get_duration(State)}
+	      transport => stun_logger:encode_transport(SockMod),
+	      reason => Reason},
+    Info2 = case {HookName, State} of
+		{turn_session_start, _State} ->
+		    Info1;
+		{turn_session_stop, #state{sent_bytes = SentBytes,
+					   sent_pkts = SentPkts,
+					   rcvd_bytes = RcvdBytes,
+					   rcvd_pkts = RcvdPkts}} ->
+		    Info1#{sent_bytes => SentBytes,
+			   sent_pkts => SentPkts,
+			   rcvd_bytes => RcvdBytes,
+			   rcvd_pkts => RcvdPkts,
+			   duration => get_duration(State)};
+		{protocol_error, _State} ->
+		    Info1
 	   end,
     ?LOG_DEBUG("Running '~s' hook", [HookName]),
-    try HookFun(HookName, Info)
+    try HookFun(HookName, Info2)
     catch _:Err -> ?LOG_ERROR("Hook '~s' failed: ~p", [HookName, Err])
     end;
-run_hook(HookName, _State) ->
+run_hook(HookName, _State, _Msg) ->
     ?LOG_DEBUG("No callback function specified for '~s' hook", [HookName]),
     ok.
 
